@@ -3,10 +3,13 @@
 ########################################################
 ##Functions in this file:
 ## createSTdata          EX:ok
-## print.STdata          EX:ok
-## summary.STdata        EX:ok
-## print.summarySTdata   EX:implicit in summary.STdata
+## print.STdata          EX:NO
+## summary.STdata        EX:NO
+## print.summarySTdata   EX:NO
 ## plot.STdata           EX:ok
+## qqnorm.STdata         EX:ok
+## scatterPlot.STdata    EX:ok
+## scatterPlot           EX:NO, S3 compliance methods
 
 ##' Creates a \code{STdata} object that can be used as input for
 ##' \code{\link{createSTmodel}}. Names and dates are derived from the input data,
@@ -30,17 +33,46 @@
 ##' @param mean.0.ST Call \code{\link{removeSTcovarMean}} to produce a mean-zero
 ##'   spatio-temporal covariate?
 ##' @param n.basis Number of temporal components in the smooth trends computed
-##'   by \code{\link{updateSTdataTrend}}, if \code{NULL} no trend is computed 
+##'   by \code{\link{updateTrend.STdata}}, if \code{NULL} no trend is computed 
 ##'   (implies only a constant).
 ##' @param extra.dates Additional dates for which smooth trends should be
-##'   computed, used by \code{\link{updateSTdataTrend}}. If \code{n.basis=NULL}
+##'   computed, used by \code{\link{updateTrend.STdata}}. If \code{n.basis=NULL}
 ##'   this will force n.basis=0; since the dates are stored in the trend..
-##' @param ... Additional parameters passed to \code{\link{updateSTdataTrend}}.
+##' @param ... Additional parameters passed to \code{\link{updateTrend.STdata}}.
 ##' @param detrend Use \code{\link{detrendSTdata}} to remove a termporal trend
 ##'   from the observations; requires \code{n.basis!=NULL}.
 ##' @param region,method Additional parameters passed to
 ##'   \code{\link{detrendSTdata}}.
-##' @return A \code{STdata} object, see \code{\link{mesa.data}} for an example.
+##' @return A \code{STdata} object with, some or all of, the following elements:
+##'   \item{covars}{Geographic covariates, locations and names
+##'     of the observation locations (the later in \code{covars$ID}),
+##'     \code{\link{createSTmodel}} will extract covariates (land use regressors),
+##'     observations locations, etc from this data.frame when constructing the
+##'     model specification.}
+##'   \item{trend}{The temporal trends with \emph{one of the} columns
+##'     being named \code{date}, preferably of class \code{\link[base:Date]{Date}}
+##'     providing the time alignment for the temporal trends.}
+##'   \item{obs}{A data.frame with columns:
+##'     \describe{
+##'       \item{obs}{The value of each observation.}
+##'       \item{date}{The observations time, preferably of class
+##'                   \code{\link[base:Date]{Date}}.}
+##'       \item{ID}{A \code{character}-class giving observation locations;
+##'                 should match elements in \code{locations$ID}.}
+##'     }
+##'   }
+##'   \item{SpatioTemporal}{A 3D-array of spatio-temporal covariates, or \code{NULL}
+##'                     if no covariates exist. The array should be
+##'                     (number of timepoints) - by - (number of locations) -
+##'                     by - (number of covariates) and provide spatio-temporal
+##'                     covariates for \emph{all} space-time locations, even
+##'                     unobserved ones (needed for prediction).
+##'                     The \code{rownames} of the array should represent dates/times
+##'                     and \code{colnames} should match the observation location
+##'                     names in \code{covars$ID}.}
+##'   \item{old.trend,fit.trend}{Additional components added if the observations
+##'                              have been detrended, see
+##'                              \code{\link{detrendSTdata}}.}
 ##' 
 ##' @example Rd_examples/Ex_createSTdata.R
 ##' 
@@ -50,13 +82,11 @@
 ##' @export
 createSTdata <- function(obs, covars, SpatioTemporal=NULL,
                          transform.obs=function(x){return(x)}, mean.0.ST=FALSE,
-                         n.basis=NULL, extra.dates=NULL, ...,
+                         n.basis=0, extra.dates=NULL, ...,
                          detrend=FALSE, region=NULL, method=NULL){
 
-  ##Check for valid inputs, if extra.dates have been specified we also need
-  ##at least n.basis=0 (to store the extra dates)
-  if( !is.null(extra.dates) && is.null(n.basis) ){
-    message("'extra.dates' implies n.basis!=NULL; taking n.basis=0")
+  ##n.basis should default to 0 if not given
+  if( is.null(n.basis) ){
     n.basis <- 0
   }
   ##Check if observations are given as a matrix or as a vector
@@ -64,6 +94,9 @@ createSTdata <- function(obs, covars, SpatioTemporal=NULL,
     ##empty observation vector
     obs <- data.frame(obs=double(0), date=integer(0), ID=character(0),
                       stringsAsFactors=FALSE)
+    if( missing(extra.dates) || is.null(extra.dates) ){
+      stop("If 'obs' is empty, 'extra.dates' is needed to define time-points")
+    }
   }else if( is.data.frame(obs) ){
     if( dim(obs)[2]<3 ){
       stop("'obs' needs at least 3 columns.")
@@ -109,6 +142,8 @@ createSTdata <- function(obs, covars, SpatioTemporal=NULL,
   }
   ##drop missing observations
   obs <- obs[!is.na(obs$obs),]
+  ##and rownames
+  rownames(obs) <- NULL
   ##transform observations
   if( is.function(transform.obs) ){
     obs$obs <- transform.obs(obs$obs)
@@ -136,10 +171,8 @@ createSTdata <- function(obs, covars, SpatioTemporal=NULL,
 
   ##now attempt to modify output object
   ##compute smooth trends?
-  if( !is.null(n.basis) ){
-    out <- updateSTdataTrend(out, n.basis=n.basis,
-                             extra.dates=extra.dates, ...)
-  }
+  out <- updateTrend(out, n.basis=n.basis, extra.dates=extra.dates, ...)
+
   ##detrend data? (will throw an internal warning if n.basis=NULL since this
   ##implies no trend)
   if( detrend ){
@@ -167,12 +200,6 @@ createSTdata <- function(obs, covars, SpatioTemporal=NULL,
 ##' @param ... Ignored additional arguments.
 ##' @return Nothing
 ##' 
-##' @examples
-##' ##load some data
-##' data(mesa.data)
-##' ##print basic information regarding obs, locations, dates, etc
-##' print(mesa.data)
-##'
 ##' @author Johan Lindstrm
 ##' 
 ##' @family STdata methods
@@ -214,12 +241,6 @@ print.STdata <- function(x, type=x$covars$type, ...){
 ##' @param ... Ignored additional arguments. 
 ##' @return A \code{summary.STdata} object.
 ##' 
-##' @examples
-##' ##load some data
-##' data(mesa.data)
-##' ##Summary of data fields.
-##' summary(mesa.data)
-##'
 ##' @author Johan Lindstrm
 ##' 
 ##' @family STdata methods
@@ -325,10 +346,22 @@ print.summary.STdata <- function(x, ...){
 ##'     \code{type} of observations locations.}
 ##' }
 ##'
+##' For \code{y=c("obs","res")} the first element of \code{col,pch,cex,lty} is
+##' used to specify plotting of the observations, and the second element is used
+##' to  specify plotting of the fitted temporal trend, or 0-line for
+##' \code{"res"}. Defaults: \code{col=1}, \code{pch=c(1,NA)}, \code{cex=1},
+##' \code{lty=c(NA,1)}. Elements of length one are repeated.
+##'
+##' For \code{y=c("acf","pacf")} \code{col,pch,cex,lty} are ignored.
+##'
+##' For \code{y=c("loc","loc.obs")} \code{col,pch,cex} are used to specify the
+##' points for each of the different levels in \code{type} and should be of
+##' length 1 or \code{length(levels(type))}. \code{lty} is ignored.
+##' Default: \code{col=1:length(levels(type))}, \code{pch=19}, \code{cex=.1}
+##' 
 ##' For \code{y=c("loc","loc.obs")} a legend is added if \code{legend.loc!=NULL}.
 ##' The vector \code{legend.names} should have length equal to the number of
 ##' unique location types.  The default legend names are \code{levels(type)}.
-##' The function uses \code{col=1:length(levels(type))} if \code{col=NULL}.
 ##' 
 ##' @title Different Plots for \code{STdata}/\code{STmodel} object
 ##' @param x \code{STdata}/\code{STmodel} object to plot.
@@ -338,21 +371,19 @@ print.summary.STdata <- function(x, ...){
 ##'   string matching the names in \code{x$covars$ID} or an integer; if
 ##'   an integer the functions will plot data from
 ##'   \code{ID=x$covars$ID[ID]}.
-##' @param type Factorial of \code{length(x$covars$ID)}, used by
-##'   \code{"loc"} and for \code{"loc.obs"} to determine how many groups should be
+##' @param type Factorial of \code{length(x$covars$type)}, used by
+##'   \code{"loc"} and \code{"loc.obs"} to determine how many groups should be
 ##'   plotted and colour/type coded.
-##' @param col,pch Colour and size of points for \code{"loc"} and
-##'   \code{"loc.obs"}, either one constant or a vector or
-##'   \code{length(levels(type))}.
-##' @param cex Size of points for \code{"loc"} and \code{"loc.obs"}.
+##' @param col,pch,cex,lty Colour, type of points, size of points, and type of
+##'   lines. Exact meaning depends on value of \code{y}, see Details.
 ##' @param legend.loc The location of the legend, for \code{"loc"} and
 ##'   \code{"loc.obs"}. See \code{\link[graphics:legend]{legend}}.
 ##' @param legend.names A vector of character strings to be used in the legend,
 ##'   for \code{"loc"} and for \code{"loc.obs"}
 ##' @param add Add to existing plot, only relevant if \code{y} is
 ##'   \code{"obs"}, \code{"res"}, \code{"loc"}, or \code{"loc.obs"}.
-##' @param ... Additional parameters passed to \code{\link[graphics:plot]{plot}},
-##'   \code{\link[stats:acf]{acf}}, or \code{\link[stats:pacf]{pacf}},
+##' @param ... Additional parameters passed to \code{\link[graphics:plot]{plot}}
+##'   or \code{\link[stats:plot.acf]{plot.acf}}.
 ##' @return Nothing
 ##'
 ##' @example Rd_examples/Ex_plot_STdata.R
@@ -362,18 +393,35 @@ print.summary.STdata <- function(x, ...){
 ##' @family STdata methods
 ##' @method plot STdata
 ##' @export
-plot.STdata <- function(x, y="obs", ID=x$covars$ID[1], type=x$covars$type,
-                        col=NULL, pch=19, cex=.1, legend.loc="topleft",
-                        legend.names=NULL, add=FALSE, ...){
+plot.STdata <- function(x, y=c("obs", "res", "acf", "pacf", "loc", "loc.obs"),
+                        ID=x$covars$ID[1], type=x$covars$type,
+                        col=NULL, pch=NULL, cex=NULL, lty=NULL,
+                        legend.loc="topleft", legend.names=NULL,
+                        add=FALSE, ...){
   ##check class belonging
   stCheckClass(x, "STdata", name="x")
 
   ##we have to use y, cast to resonable name
-  plot.type <- y
-
-  if( !(plot.type %in% c("obs", "res", "acf", "pacf", "loc", "loc.obs")) ){
-    stop("Unknown option for 'y' should be obs, res, acf, pacf, loc, loc.obs")
+  plot.type <- match.arg(y)
+  
+  ##defualt values of col,pch,cex,lty
+  if( (plot.type %in% c("obs", "res")) ){
+    if( missing(col) || is.null(col) ){ col <- 1 }
+    if( missing(pch) || is.null(pch) ){ pch <- c(1,NA) }
+    if( missing(cex) || is.null(cex) ){ cex <- 1 }
+    if( missing(lty) || is.null(lty) ){ lty <- c(NA,1) }
+    ##expand to length=2
+    if( length(col)==1 ){ col <- rep(col,2) }
+    if( length(pch)==1 ){ pch <- rep(pch,2) }
+    if( length(cex)==1 ){ cex <- rep(cex,2) }
+    if( length(lty)==1 ){ lty <- rep(lty,2) }
+    
+  }else if( (plot.type %in% c("loc", "loc.obs")) ){
+    if( missing(pch) || is.null(pch) ){ pch <- 19 }
+    if( missing(cex) || is.null(cex) ){ cex <- 0.1 }
+    ##lty ignored, and defaults for col=NULL handled below
   }
+  
   if( (plot.type %in% c("obs", "res", "acf", "pacf")) ){
     ##previous plotMesaData function
     if( length(ID)!=1 ){
@@ -407,57 +455,70 @@ plot.STdata <- function(x, y="obs", ID=x$covars$ID[1], type=x$covars$type,
     }##if( is.null(x$trend) ){...}else{...}
     N.trend <- length(date.trend)
     
+    ##linear regression
     if( is.null(trend) ){
-      ##just a constant temporal trend 
-      if( plot.type!="obs" ){
-        ##if not observations we only need the residuals
-        y <- y - mean(y,na.rm=TRUE)
-        y.p <- NULL
-      }else{
-        ##prediction of y for all times
-        y.p <- rep(mean(y,na.rm=TRUE), N.trend)
-      }
+      y.fit <- lm(y~1)
     }else{
-      ##linear regression
-      y.fit <- lm(y~as.matrix(trend))
-      if( plot.type!="obs" ){
-        ##if not observations we only need the residuals
-        y <- y.fit$residuals
-        y.p <- NULL
+      y.fit <- lm(paste("y~", paste(colnames(trend), collapse="+")),
+                  data=cbind(y,trend))
+    }
+    if( plot.type!="obs" ){
+      ##if not observations we only need the residuals
+      y <- y.fit$residuals
+      y.p <- rep(0, length(date.trend))
+    }else{
+      ##prediction of y for all times (works also for empty trend)
+      suppressWarnings( y.p <- predict(y.fit, x$trend) )
+    }
+    if( y.fit$df.residual==0 ){
+      if(plot.type %in% c("acf","pacf")){
+        warning("Site ",ID," has too few observations, acf/pacf may FAIL.")
       }else{
-        ##prediction of y for all times
-        y.p <- (as.matrix(x$trend[, -which(names(x$trend)=="date"),drop=FALSE]) %*%
-                y.fit$coefficients[-1] + y.fit$coefficients[1])
+        warning("Site ",ID," has too few observations, fitted smooths may be inacurate.")
       }
-    }##if( is.null(trend) ){...}else{...}
+    }
     
     if(plot.type %in% c("acf","pacf")){
       ##plot correlation functions
-      ##expand the dates
-      date.exp <- seq(min(date),max(date),min(diff(date)))
+      ##expand the dates to equidistant locations
+      date.exp <- seq(min(date), max(date), min(diff(date.trend)))
       y.exp <- rep(NA, length(date.exp))
       ##and match the dates suitably.
       y.exp[match(date,date.exp)] <- y
+      ##construct list of arguments
+      args <- internalPlotFixArgs(list(...),
+                                  default=list(main=switch(plot.type,
+                                                 "acf"=paste("ACF for",ID),
+                                                 "pacf"=paste("PACF for",ID))),
+                                  add=list(x=y.exp, na.action=na.pass))
       if(plot.type == "acf"){
-        acf(y, main=paste("ACF for",ID), ...)
+        do.call(acf, args)
       }else{
-        pacf(y, main=paste("PACF for",ID), ...)
+        do.call(pacf, args)
       }
     }else{
       ##plot time series or residuals
-      if(add){
-        points(date, y)
-      }else{
-        plot(date, y, xlim = range(c(date,x$trend$date),na.rm=TRUE),
-             ylim = range(c(y,y.p),na.rm=TRUE), 
-             main=switch(plot.type,"obs"=ID,"res"=paste("Residuals",ID)),
-             ...)
+      if(!add){
+        args <- internalPlotFixArgs(list(...),
+                                    default=list(xlab="date", ylab="y",
+                                      main=switch(plot.type, "obs"=ID,
+                                        "res"=paste("Residuals",ID))),
+                                    add=list(x=date, y=y, xlim =
+                                      range(c(date,x$trend$date),na.rm=TRUE),
+                                      ylim = range(c(y,y.p),na.rm=TRUE),
+                                      type="n"))
+        do.call(plot, args)
       }
-      if(plot.type=="obs"){
-        lines(date.trend, y.p)
-      }else{
-        abline(h=0, col="grey")
+      ##add points and lines
+      if( !is.na(pch[1]) ){
+        points(date, y, pch=pch[1], cex=cex[1], col=col[1])
       }
+      if( !is.na(lty[1]) ){ lines(date, y, lty=lty[1], col=col[1]) }
+      ##plot observations or residuals (different fitted lines)
+      if( !is.na(pch[2]) ){
+        points(date.trend, y.p, pch=pch[2], col=col[2], cex=cex[2])
+      }
+      if( !is.na(lty[2]) ){ lines(date.trend, y.p, lty=lty[2], col=col[2]) }
     }##if(plot.type %in% c("acf","pacf")){...}else{...}
   }else{
     ##previous plotMonitoringLoc function
@@ -473,19 +534,12 @@ plot.STdata <- function(x, y="obs", ID=x$covars$ID[1], type=x$covars$type,
     idx <- match( x$obs$ID, x$covars$ID)
     
     ##set up colours
-    if( is.null(col) ){
-      col <- 1:length(levels(type))
-    }
-    if( length(col)==1 ){
-      col <- rep(col, length(levels(type)))
-    }
-    ##...and point-types
-    if( is.null(pch) ){
-      pch <- 19
-    }
-    if( length(pch)==1 ){
-      pch <- rep(pch, length(levels(type)))
-    }
+    if( is.null(col) ){ col <- 1:length(levels(type)) }
+    if( length(col)==1 ){ col <- rep(col, length(levels(type))) }
+    ##point-types
+    if( length(pch)==1 ){ pch <- rep(pch, length(levels(type))) }
+    ##...and cex-types
+    if( length(cex)==1 ){ cex <- rep(cex, length(levels(type))) }
     ##set up legend
     if( !is.null(legend.loc) ){
       if( is.null(legend.names) ){
@@ -507,22 +561,135 @@ plot.STdata <- function(x, y="obs", ID=x$covars$ID[1], type=x$covars$type,
       y.text <- "Location ID"
     }
     if(!add){
-      plot(x$obs$date, y.vals, type="n", xlab="Date", ylab=y.text, ...)
+      args <- internalPlotFixArgs(list(...),
+                                  default=list(xlab="Date", ylab=y.text),
+                                  add=list(x=x$obs$date, y=y.vals, type="n"))
+      do.call(plot, args)
     }
     ##loop over all possible types
     j=1
     for(i in levels(type) ){
       Ind <- (idx %in% which(type==i))
       if( sum(Ind)!=0 ){
-        points(x$obs$date[Ind], y.vals[Ind], col=col[j], pch=pch[j], cex=cex)
+        points(x$obs$date[Ind], y.vals[Ind], col=col[j], pch=pch[j], cex=cex[j])
       }
       j=j+1
     }
     ##possibly add a legend
     if( !is.null(legend.loc) ){
-      legend(legend.loc, legend.names, pch=pch, col=col, pt.cex=.5)
+      legend(legend.loc, legend.names, pch=pch, col=col, pt.cex=cex)
     }
   }##if( (plot.type %in% c("obs", "res", "acf", "pacf")) ){...}else{...}
   
   return(invisible())
 }##function plot.STdata
+
+##' \code{\link[stats:qqnorm]{qqnorm}} method for classes
+##' \code{STdata}/\code{STmodel}/\code{predCVSTmodel}. 
+##' Used for data and residual analysis of the cross validation.
+##' 
+##' @title QQ-norm for \code{STdata}/\code{STmodel}/\code{predCVSTmodel} objects
+##' @param y \code{STdata}/\code{STmodel}/\code{predCVSTmodel} object for the
+##'   qqnorm.
+##' @param ID The location for which we want to norm-plot observations/residuals
+##'   or \code{"all"} to plot for all locations.
+##' @param main Title of the plot
+##' @param group Do the norm-plot both for all data and then for each subset
+##'   defined by the factor/levels in group variable.
+##' @param col Colour of points in the plot, either a scalar or a vector
+##'   with length matching the number of observations/residuals.
+##' @param line If non-zero add a \code{\link[stats:qqline]{qqline}} with
+##'   \code{lty=line}, to the plot; if 0 \emph{do not} add a line.
+##' @param ...  Arguments passed on to the plotting function,
+##'   \code{\link[stats:qqnorm]{qqnorm}}. 
+##' @return Nothing
+##'
+##' @author Johan Lindstrm
+##' 
+##' @example Rd_examples/Ex_qqnorm_STdata.R
+##' 
+##' @family STdata methods
+##' @importFrom stats qqnorm
+##' @method qqnorm STdata
+##' @export
+qqnorm.STdata <- function(y, ID="all", main="Q-Q plot for observations",
+                          group=NULL, col=1, line=0, ...){ 
+  ##check class belonging
+  stCheckClass(y, "STdata", name="y")
+
+  Y <- y$obs[, c("ID","obs")]
+
+  internalQQnormPlot(Y, ID, main, group, col, FALSE, line, ...)
+  
+  return(invisible())
+}##function qqnorm.STdata
+
+##' Does a scatterPlot of observations/residuals against covariates (either
+##' geographic or temporal trends), adding a spline fit (similar to
+##' \code{\link[stats:scatter.smooth]{scatter.smooth}}.
+##'
+##' @title scatterPlot for \code{STdata}/\code{STmodel}/\code{predCVSTmodel} objects
+##' 
+##' @param x \code{STdata}/\code{STmodel}/\code{predCVSTmodel} object to plot.
+##' @param covar,trend Plot observations as a function of? Only \emph{one} of
+##'   these should be not \code{NULL}. \code{covar} uses location covariates,
+##'   and \code{trend} uses temporal trend (or dates); \code{trend=0} uses a
+##'   temporal intercept (i.e. a constant).
+##' @param pch,cex Point and point size for the plot, a single value or
+##'   \code{nlevels(group)} 
+##' @param col,lty Color of points and smooth lines. A single value or
+##'  \code{nlevels(group)+1} values; the last value is used for fitting a line
+##'  to \emph{all} data. Use \code{lty=NA} to supress smooth lines.
+##' @param subset A subset of locations for which to plot observations as a
+##'   function of covariates.
+##' @param group A vector of factors of the same length as the number of
+##'   observations (typically \code{length(x$obs$obs)}, or
+##'   \code{length(x$pred.obs$obs)}) used to group data and fit different
+##'   smooths to each group.
+##' @param add Add to existing plot
+##' @param smooth.args List of arguments for
+##'   \code{\link[stats:loess.smooth]{loess.smooth}}.
+##' @param ... Additional parameters passed to \code{\link[graphics:plot]{plot}}.
+##' @return Nothing
+##'
+##' @example Rd_examples/Ex_scatterPlot_STdata.R
+##' 
+##' @author Johan Lindstrm
+##' @family STdata methods
+##' @method scatterPlot STdata
+##' @export
+scatterPlot.STdata <- function(x, covar=NULL, trend=NULL, pch=1, col=1, cex=1,
+                               lty=1, subset=NULL, group=NULL, add=FALSE,
+                               smooth.args=NULL, ...){
+  ##check class belonging
+  stCheckClass(x, "STdata", name="x")
+
+  ##add trend to avoid future problems
+  if( is.null(x$trend) ){
+    x <- updateTrend(x, n.basis=0)
+  }
+  
+  ##pass data to internalScatterPlot function
+  internalScatterPlot(obs=x$obs[, c("obs","ID","date")],
+                      covar=covar, trend=trend, subset=subset,
+                      data=list(covars=x$covars, trend=x$trend),
+                      group=group, pch=pch, col=col, cex=cex, lty=lty,
+                      add=add, smooth.args=smooth.args, ...) 
+}##function scatterPlot.STdata
+
+################################
+## S3 methods for scatterPlot ##
+################################
+
+##' Scatter plot of data in x
+##'
+##' @title Scatter plot
+##' @param x object to scatter plot
+##' @param ... additional parameters
+##' @return Nothing
+##' 
+##' @author Johan Lindstrm
+##' @export
+scatterPlot <- function(x, ...){
+  UseMethod("scatterPlot")
+}
