@@ -1,9 +1,12 @@
 ##load the data
-data(mesa.data)
+data(mesa.data.raw)
+##and create STdata-object
+mesa.data <- createSTdata(mesa.data.raw$obs, mesa.data.raw$X, n.basis=2,
+                          SpatioTemporal=mesa.data.raw["lax.conc.1500"])
 
 ##define land-use covariates
-LUR <-  list(c("log10.m.to.a1", "s2000.pop.div.10000", "km.to.coast"),
-             "km.to.coast", "km.to.coast")
+LUR <-  list(~log10.m.to.a1+s2000.pop.div.10000+km.to.coast,
+             ~km.to.coast, ~km.to.coast)
 ##and covariance model
 cov.beta <- list(covf="exp", nugget=FALSE)
 cov.nu <- list(covf="exp", nugget=TRUE, random.effect=FALSE)
@@ -11,8 +14,9 @@ cov.nu <- list(covf="exp", nugget=TRUE, random.effect=FALSE)
 locations <- list(coords=c("x","y"), long.lat=c("long","lat"), others="type")
 
 ##create object
-mesa.model <- createSTmodel(mesa.data, LUR=LUR, cov.beta=cov.beta,
-                            cov.nu=cov.nu, locations=locations)
+mesa.model <- createSTmodel(mesa.data, LUR=LUR, ST="lax.conc.1500",
+                            cov.beta=cov.beta, cov.nu=cov.nu,
+                            locations=locations)
 print(mesa.model)
 ##This is the same as data(mesa.model)
 
