@@ -1,5 +1,8 @@
 ##load the data
-data(mesa.data)
+data(mesa.data.raw)
+##and create STdata-object
+mesa.data <- createSTdata(mesa.data.raw$obs, mesa.data.raw$X, n.basis=2,
+                          SpatioTemporal=mesa.data.raw["lax.conc.1500"])
 
 ##plot time-series for the first site,
 par(mfrow=c(3,2),mar=c(2.5,2.5,3,1))
