@@ -1,27 +1,21 @@
 ##load a model object
 data(mesa.model)
 
-##examine the model
-print(mesa.model)
-
-##Important dimensions of the model
+##create vector of initial values
 dim <- loglikeSTdim(mesa.model)
-print(dim)
-
-##Set up initial parameter values for optimization
-x.init <- cbind(rep(2,dim$nparam.cov), c(rep(c(1,-3),dim$m+1),-3))
-##and add names to the initial values
+x.init <- cbind(c( rep(2, dim$nparam.cov-1), 0),
+                c( rep(c(1,-3), dim$m+1), -3, 0))
 rownames(x.init) <- loglikeSTnames(mesa.model, all=FALSE)
-print(x.init)
+
 \dontrun{
   ##estimate parameters
-  ##This may take a while...
-  est.mesa.model <- estimate(mesa.model, x.init, type="p", hessian.all=TRUE)
+  est.mesa.model <- estimate(mesa.model, x.init, hessian.all=TRUE)
 }
-##Let's load precomputed results instead.
+
+##time consuming estimation, load pre-computed results instead
 data(est.mesa.model)
 
-##Optimisation status message
+#estimation results
 print(est.mesa.model)
 
 ##compare the estimated parameters for the two starting points
@@ -32,16 +26,20 @@ est.mesa.model$summary$status
 ##extract the estimated parameters and approximate uncertainties
 x <- coef(est.mesa.model)
 
+##compare estimated parameters
 ##plot the estimated parameters with uncertainties
 par(mfrow=c(1,1),mar=c(13.5,2.5,.5,.5))
-plot(x$par, ylim=range(c( x$par-1.96*x$sd, x$par+1.96*x$sd )),
-     xlab="", xaxt="n")
-points(x$par - 1.96*x$sd, pch=3)
-points(x$par + 1.96*x$sd, pch=3)
+with(x, plot(par, ylim=range(c(par-1.96*sd, par+1.96*sd)),
+                xlab="", xaxt="n"))
+with(x, points(par - 1.96*sd, pch=3))
+with(x, points(par + 1.96*sd, pch=3))
+
+abline(h=0, col="grey")
+##add axis labels
 axis(1, 1:length(x$par), rownames(x), las=2)
 
 \dontrun{
-  ##fixed pars
+  ##example using a few fixed parameters
   x.fixed <- coef(est.mesa.model)$par
   x.fixed[c(1,2,5:9)] <- NA
   est.fix <- estimate(mesa.model, x.init, x.fixed, type="p")
