@@ -137,31 +137,31 @@ void iid_cov(int n, double* par, double* dist, double* cov){
 /* COMMON SETS OF NAMES FOR COVARIANCE PARAMETERS */
 //no parameters (e.g. iid)
 SEXP cov_names_none(){
-   SEXP parnames;           
-   PROTECT( parnames=allocVector(STRSXP,0) );
-   UNPROTECT(1);
-   return parnames;
+  SEXP parnames;           
+  PROTECT( parnames=allocVector(STRSXP,0) );
+  UNPROTECT(1);
+  return parnames;
 }
 
 //just range and sill (e.g. exp, exp2/gaussian, cubic, spherical
 SEXP cov_names_range_sill(){
-   SEXP parnames;           
-   PROTECT( parnames=allocVector(STRSXP,2) );
-   SET_STRING_ELT(parnames, 0, mkChar("range"));
-   SET_STRING_ELT(parnames, 1, mkChar("sill"));
-   UNPROTECT(1);
-   return parnames;
+  SEXP parnames;           
+  PROTECT( parnames=allocVector(STRSXP,2) );
+  SET_STRING_ELT(parnames, 0, mkChar("range"));
+  SET_STRING_ELT(parnames, 1, mkChar("sill"));
+  UNPROTECT(1);
+  return parnames;
 }
 
 //range, sill, and shape (e.g. matern, cauchy)
 SEXP cov_names_range_sill_shape(){
-   SEXP parnames;           
-   PROTECT( parnames=allocVector(STRSXP,3) );
-   SET_STRING_ELT(parnames, 0, mkChar("range"));
-   SET_STRING_ELT(parnames, 1, mkChar("sill"));
-   SET_STRING_ELT(parnames, 2, mkChar("shape"));
-   UNPROTECT(1);
-   return parnames;
+  SEXP parnames;           
+  PROTECT( parnames=allocVector(STRSXP,3) );
+  SET_STRING_ELT(parnames, 0, mkChar("range"));
+  SET_STRING_ELT(parnames, 1, mkChar("sill"));
+  SET_STRING_ELT(parnames, 2, mkChar("shape"));
+  UNPROTECT(1);
+  return parnames;
 }
 
 /* SELECTION OF COVARIANCE FUNCTION */
@@ -181,7 +181,7 @@ cov_funs_struct selectCov(const char* cov_type){
     cov_funs.cov = exp_cov;
     cov_funs.cov_names = cov_names_range_sill;
   }else if( strcmp(cov_type, "exp2")==0 || strcmp(cov_type, "exponential2")==0 ||
-	    strcmp(cov_type, "gaussian")==0 ){
+            strcmp(cov_type, "gaussian")==0 ){
     //double exponential / Gaussian
     cov_funs.cov = exp2_cov;
     cov_funs.cov_names = cov_names_range_sill;
@@ -221,8 +221,8 @@ cov_funs_struct selectCov(const char* cov_type){
 //  offset after each column in the distance matrix, (n.blocks-1)*n1
 //  indicator if the matrix is symmetric.
 void computeCovarianceBlock(cov_funs_struct cov_funs, double* pars,
-			    double random, double* dist, double* covf,
-			    int n1, int n2, int n_offset, int symmetric){
+                            double random, double* dist, double* covf,
+                            int n1, int n2, int n_offset, int symmetric){
   //loop variables.
   int i, j;
   //total offset is n1+n_offset
@@ -238,7 +238,7 @@ void computeCovarianceBlock(cov_funs_struct cov_funs, double* pars,
     //Use symmetry to fill in the lower left half
     for(i=0; i<n2; ++i){
       for(j=i+1; j<n1; ++j){
-	covf[j+i*n_offset_tot] = covf[i+j*n_offset_tot];
+        covf[j+i*n_offset_tot] = covf[i+j*n_offset_tot];
       }
     }
   }else if( n_offset!=0 ){
@@ -257,7 +257,7 @@ void computeCovarianceBlock(cov_funs_struct cov_funs, double* pars,
   if( random!=0 ){
     for(i=0; i<n2; ++i){
       for(j=0; j<n1; ++j){
-	covf[j+i*n_offset_tot] += random;
+        covf[j+i*n_offset_tot] += random;
       }
     }
   }//if( random!=0 )
@@ -267,25 +267,25 @@ void computeCovarianceBlock(cov_funs_struct cov_funs, double* pars,
 /* R INTERFACE FUNCTIONS FOR TYPE OF COVARIANCE/PARAMETER NAMES*/
 //first, return possible names of covariance functions
 SEXP C_cov_names(){
-   SEXP parnames;
-   //increase the number to account for additional covariance names
-   int n_covs = 10;
-   PROTECT( parnames=allocVector(STRSXP,n_covs) );
-   //add name(s) to the end of the list
-   SET_STRING_ELT(parnames, 0, mkChar("exp"));
-   SET_STRING_ELT(parnames, 1, mkChar("exponential"));
-   SET_STRING_ELT(parnames, 2, mkChar("exp2"));
-   SET_STRING_ELT(parnames, 3, mkChar("exponential2"));
-   SET_STRING_ELT(parnames, 4, mkChar("gaussian"));
-   SET_STRING_ELT(parnames, 5, mkChar("cubic"));
-   SET_STRING_ELT(parnames, 6, mkChar("spherical"));
-   SET_STRING_ELT(parnames, 7, mkChar("matern"));
-   SET_STRING_ELT(parnames, 8, mkChar("cauchy"));
-   SET_STRING_ELT(parnames, 9, mkChar("iid"));
+  SEXP parnames;
+  //increase the number to account for additional covariance names
+  int n_covs = 10;
+  PROTECT( parnames=allocVector(STRSXP,n_covs) );
+  //add name(s) to the end of the list
+  SET_STRING_ELT(parnames, 0, mkChar("exp"));
+  SET_STRING_ELT(parnames, 1, mkChar("exponential"));
+  SET_STRING_ELT(parnames, 2, mkChar("exp2"));
+  SET_STRING_ELT(parnames, 3, mkChar("exponential2"));
+  SET_STRING_ELT(parnames, 4, mkChar("gaussian"));
+  SET_STRING_ELT(parnames, 5, mkChar("cubic"));
+  SET_STRING_ELT(parnames, 6, mkChar("spherical"));
+  SET_STRING_ELT(parnames, 7, mkChar("matern"));
+  SET_STRING_ELT(parnames, 8, mkChar("cauchy"));
+  SET_STRING_ELT(parnames, 9, mkChar("iid"));
    
-   //release memory back to R
-   UNPROTECT(1);
-   return parnames;
+  //release memory back to R
+  UNPROTECT(1);
+  return parnames;
 }
 
 //second, given a string, return expected names of parameters
@@ -304,7 +304,7 @@ SEXP C_cov_pars(SEXP cov_typeR){
 
   if( cov_funs.cov==NULL || cov_funs.cov_names==NULL ){
     //bad name of covariance function, return NULL
-   return R_NilValue;
+    return R_NilValue;
   }
   //else return names assigned above
   return cov_funs.cov_names();
@@ -322,16 +322,16 @@ int max_create_cov(int* vec, int n){
 }
 
 void checkCovarianceFunAndPars(cov_funs_struct cov_funs, const char* cov_type,
-			       int n_pars){
+                               int n_pars){
   //valid covariance function?
   if( cov_funs.cov==NULL || cov_funs.cov_names==NULL ){
     error("Unknown covariance specification: 'type' = %s",
-	  cov_type);
+          cov_type);
   }
   //check length of parameter vector
   if( length(cov_funs.cov_names())!=n_pars ){
     error("Expected %d parameter(s), but length(pars) = %d",
-	  n_pars, length(cov_funs.cov_names()));
+          n_pars, length(cov_funs.cov_names()));
   }
 }//checkCovarianceFunAndPars(...)
 
@@ -369,7 +369,7 @@ SEXP C_cov_simple(SEXP cov_typeR, SEXP parsR, SEXP distR){
 
 //compute block cross-covariance matrix for Sigma_B
 SEXP C_makeSigmaB(SEXP cov_typeR, SEXP parsR, SEXP nuggetR,
-		  SEXP distR, SEXP symmetryR, SEXP ind2_to_1R){
+                  SEXP distR, SEXP symmetryR, SEXP ind2_to_1R, SEXP sparseR){
   //EXTRACT PARAMETERS FROM R CALL
   int n_blocks = length(cov_typeR);
   double* nugget = REAL(nuggetR);
@@ -380,6 +380,8 @@ SEXP C_makeSigmaB(SEXP cov_typeR, SEXP parsR, SEXP nuggetR,
   int n2 = INTEGER(matDim)[1];
   //is matrix symmetric
   int symmetry = *INTEGER(symmetryR);
+  //return sparse form (i.e. list of blocks)?
+  int sparse = *INTEGER(sparseR);
   //the index vector that maps locations in the second dimension to
   //first dimension locations, used for non-symmetric case to determine
   //if we need a nugget
@@ -390,11 +392,12 @@ SEXP C_makeSigmaB(SEXP cov_typeR, SEXP parsR, SEXP nuggetR,
   cov_funs_struct cov_funs[n_blocks];
 
   //loop variables
-  int i, j, k, offset, col_offset;
+  int i, j, k, offset, col_offset, n_blocks_tmp;
 
   //return variables
-  double *result;
-  SEXP resultR;
+  double *result, *result_all;
+  double* result_part[n_blocks];
+  SEXP resultR, result_partR;
 
   //check length of the inputs
   if( length(parsR)!=n_blocks || length(nuggetR)!=n_blocks ){
@@ -415,49 +418,76 @@ SEXP C_makeSigmaB(SEXP cov_typeR, SEXP parsR, SEXP nuggetR,
     pars[i] = REAL( VECTOR_ELT(parsR, i) );
     //check that we're getting the right number of parameters
     checkCovarianceFunAndPars(cov_funs[i], CHAR(STRING_ELT(cov_typeR,i)),
-			      length(VECTOR_ELT(parsR, i)) );
+                              length(VECTOR_ELT(parsR, i)) );
   }
-  //allocate memory for return matrix
-  PROTECT(resultR = allocMatrix(REALSXP, n1*n_blocks, n2*n_blocks));
-  result = REAL(resultR);
-  //initialise to zero
-  memset(result, 0, n1*n2*n_blocks*n_blocks*sizeof(double) );
+
+  if( sparse!=0 ){
+    //allocate memory for return list of matrices
+    PROTECT(resultR = allocVector(VECSXP, n_blocks));
+    for(i=0; i<n_blocks; ++i){
+      PROTECT(result_partR = allocMatrix(REALSXP, n1, n2));
+      SET_VECTOR_ELT(resultR, i, result_partR);
+      result_part[i] = REAL(result_partR);
+      //initialise to zero
+      memset(result_part[i], 0, n1*n2*sizeof(double) );
+    }
+    //colum offset is zero since we only do one block at a time
+    col_offset = 0;
+    n_blocks_tmp = 1;
+  }else{
+    //allocate memory for return matrix
+    PROTECT(resultR = allocMatrix(REALSXP, n1*n_blocks, n2*n_blocks));
+    result_all = REAL(resultR);
+    //initialise to zero
+    memset(result_all, 0, n1*n2*n_blocks*n_blocks*sizeof(double) );
+    //colum offset
+    col_offset = n1*(n_blocks-1);
+    n_blocks_tmp = n_blocks;
+  }//if( sparse!=0 ){...}else{...}
 
   //loop over the blocks
-  col_offset = n1*(n_blocks-1);
   for(i=0; i<n_blocks; ++i){
-    offset = i*(n1*n2*n_blocks + n1);
-    computeCovarianceBlock(cov_funs[i], pars[i], 0, dist,
-			   result + offset, n1, n2, col_offset,
-			   symmetry);
+    if( sparse!=0 ){
+      result = result_part[i];
+      offset = 0;
+    }else{
+      result = result_all;
+      offset = i*(n1*n2*n_blocks + n1);
+    }
+    computeCovarianceBlock(cov_funs[i], pars[i], 0, dist, result + offset,
+                           n1, n2, col_offset, symmetry);
     //add nugget (if non-zero)
     if( nugget[i]!=0 ){
       if( symmetry!=0 ){
-	//symmetric add to all the diagonals
-	for(j=0; j<n1; ++j){
-	  result[offset + j*n1*n_blocks + j] += nugget[i];
-	}
+        //symmetric add to all the diagonals
+        for(j=0; j<n1; ++j){
+          result[offset + j*n1*n_blocks_tmp + j] += nugget[i];
+        }
       }else{
-	//non-symmetric, use ind2_to_1 to determine nugget
-	for(k=0; k<n2; ++k){
-	  for(j=0; j<n1; ++j){
-	    if( j==(ind2_to_1[k]-1) ){
-	      result[offset + k*n1*n_blocks + j] += nugget[i];
-	    }
-	  }
-	}
+        //non-symmetric, use ind2_to_1 to determine nugget
+        for(k=0; k<n2; ++k){
+          for(j=0; j<n1; ++j){
+            if( j==(ind2_to_1[k]-1) ){
+              result[offset + k*n1*n_blocks_tmp + j] += nugget[i];
+            }
+          }
+        }
       }//if( symmetry!=0 ){...}else{...}
     }//if( nugget[i]!=0 )
   }//for(i=0; i<n_blocks; ++i)
   
-  UNPROTECT(1); /* resultR */
+  if( sparse!=0 ){
+    UNPROTECT(1+n_blocks); /* resultR + list components*/
+  }else{
+    UNPROTECT(1); /* resultR */
+  }
   return resultR;
 }//C_makeSigmaB(...)
 
 //compute block cross-covariance matrix for Sigma_nu
 SEXP C_makeSigmaNu(SEXP cov_typeR, SEXP parsR, SEXP nuggetR, SEXP randomR,
-		   SEXP distR, SEXP blocks1R, SEXP blocks2R, SEXP ind1R,
-		   SEXP ind2R, SEXP ind2_to_1R, SEXP symmetryR){
+                   SEXP distR, SEXP blocks1R, SEXP blocks2R, SEXP ind1R,
+                   SEXP ind2R, SEXP ind2_to_1R, SEXP symmetryR, SEXP sparseR){
   //EXTRACT PARAMETERS FROM R CALL
   //get covariance function name from R variable
   const char* cov_type = CHAR( STRING_ELT(cov_typeR,0) );
@@ -488,6 +518,8 @@ SEXP C_makeSigmaNu(SEXP cov_typeR, SEXP parsR, SEXP nuggetR, SEXP randomR,
   int* ind2_to_1 = INTEGER(ind2_to_1R);
   //is matrix symmetric
   int symmetry = *INTEGER(symmetryR);
+  //return sparse form (i.e. list of blocks)?
+  int sparse = *INTEGER(sparseR);
   
   //loop variables
   int i, j, i_block;
@@ -498,7 +530,7 @@ SEXP C_makeSigmaNu(SEXP cov_typeR, SEXP parsR, SEXP nuggetR, SEXP randomR,
   
   //return variables
   double *result, *covf;
-  SEXP resultR;
+  SEXP resultR, result_partR;
 
   //check dimensions of cov_type, random_effect and nugget
   if( length(cov_typeR)!=1 || length(randomR)!=1 ){
@@ -544,59 +576,80 @@ SEXP C_makeSigmaNu(SEXP cov_typeR, SEXP parsR, SEXP nuggetR, SEXP randomR,
   //Compute the (cross-)covariance matrix for all locations.
   covf = Calloc(n1*n2, double);
   computeCovarianceBlock(cov_funs, pars, random, dist,
-			 covf, n1, n2, 0, symmetry);
+                         covf, n1, n2, 0, symmetry);
   //add nugget (unless length=1 and 0)
   if( !(n_nugget!=1 && nugget[0]==0) ){
     if( symmetry!=0 ){
       //symmetric add to the diagonal
       for(i=0; i<n1; ++i){
-	if( n_nugget==1 ){
-	  covf[i*n1 + i] += nugget[0];
-	}else{
-	  covf[i*n1 + i] += nugget[i];
-	}
+        if( n_nugget==1 ){
+          covf[i*n1 + i] += nugget[0];
+        }else{
+          covf[i*n1 + i] += nugget[i];
+        }
       }//for(i=0; i<n1; ++i){
     }else{
       //non-symmetric, use ind2_to_1 to determine nugget
       for(i=0; i<n2; ++i){
-	for(j=0; j<n1; ++j){
-	  if( j==(ind2_to_1[i]-1) ){
-	    if( n_nugget==1 ){
-	      covf[i*n1 + j] += nugget[0];
-	    }else{
-	      covf[i*n1 + j] += nugget[j];
-	    }
-	  }
-	}//for(j=0; j<n1; ++j)
+        for(j=0; j<n1; ++j){
+          if( j==(ind2_to_1[i]-1) ){
+            if( n_nugget==1 ){
+              covf[i*n1 + j] += nugget[0];
+            }else{
+              covf[i*n1 + j] += nugget[j];
+            }
+          }
+        }//for(j=0; j<n1; ++j)
       }//for(i=0; i<n2; ++i)
     }//if( symmetry!=0 ){...}else{...}
   }//if( !(n_nugget!=1 && nugget[0]==0) ){
 
   //allocate memory for return matrix
-  PROTECT(resultR = allocMatrix(REALSXP, N1, N2));
-  result = REAL(resultR);
-  memset(result, 0, N1*N2*sizeof(double));
+  if( sparse!=0 ){
+    //allocate memory for return list of matrices
+    PROTECT(resultR = allocVector(VECSXP, n_blocks));
+  }else{
+    PROTECT(resultR = allocMatrix(REALSXP, N1, N2));
+    result = REAL(resultR);
+    memset(result, 0, N1*N2*sizeof(double));
+  }
 
   //loop over the blocks
   for(i_block=0; i_block<n_blocks; ++i_block){
     //size of the current block
     n1_block = blocks1[i_block];
     n2_block = blocks2[i_block];
+    
+    if( sparse!=0 ){
+      //allocate matrix block for this part
+      PROTECT(result_partR = allocMatrix(REALSXP, n1_block, n2_block));
+      SET_VECTOR_ELT(resultR, i_block, result_partR);
+      result = REAL(result_partR);
+      memset(result, 0, n1_block*n2_block*sizeof(double));
+      N1 = n1_block;
+    }
+    
     for(i=0; i<n1_block; ++i){
       for(j=0; j<n2_block; ++j){
-	//add relevant element to the large covariance matrix,
-	//recall that ind1 and ind2 are base 1 indecies!
-	result[i + j*N1] = covf[(ind1[i]-1) + (ind2[j]-1)*n1];
+        //add relevant element to the large covariance matrix,
+        //recall that ind1 and ind2 are base 1 indecies!
+        result[i + j*N1] = covf[(ind1[i]-1) + (ind2[j]-1)*n1];
       }
     }
     //incremeant pointers to account for the i:th block
     ind1 += n1_block;
     ind2 += n2_block;
-    result += n1_block + N1*n2_block;
+    if( sparse==0 ){
+      result += n1_block + N1*n2_block;
+    }
   }
   
   //free temporary covariance matrix
   Free(covf);
-  UNPROTECT(1); /* resultR */
+  if( sparse!=0 ){
+    UNPROTECT(1+n_blocks); /* resultR + list components*/
+  }else{
+    UNPROTECT(1); /* resultR */
+  }
   return resultR;
 }//C_makeSigmaNu(...)
